/***************************************************************************
                          sqlguidoc.cpp  -  description
                             -------------------
    begin                : Sat Apr 14 18:28:06 CEST 2001
    copyright            : (C) 2001 by mb
    email                : marcus@beranek.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for Qt
#include <qdir.h>
#include <qwidget.h>

// include files for KDE
#include <klocale.h>
#include <kmessagebox.h>
#include <kio/job.h>
//#include <kio/netaccess.h>

// application specific includes
#include "sqlguidoc.h"
#include "sqlgui.h"
#include "sqlguiview.h"


#include <iostream>
#include <iomanip>

QList<SqlGuiView> *SqlGuiDoc::pViewList = 0L;


SqlGuiDoc::SqlGuiDoc(QWidget *parent, const char *name) : QObject(parent, name)
{
  if(!pViewList)
  {
    pViewList = new QList<SqlGuiView>();
  }

  pViewList->setAutoDelete(true);
  
  DB = NULL;
  RESULT = NULL;
  RESULT_Start = NULL;

  doc_host.setLatin1("");
  doc_uname.setLatin1("");
  doc_pass.setLatin1("");

  QUERY     = new QString;
  STATUS    = new QString;

  // gibt es in mysql eine String-Begrenzung fr DB-Name etc. auf 256 Zeichen?
  // falls nicht, mu hier noch nachgebessert werden...
  lastDB    = new QString("XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX");
  lastTable = new QString("XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX");
  primKey   = new QString("XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX");

	lastDB->setLatin1("");
	lastTable->setLatin1("");
	primKey->setLatin1("");

  connect( this, SIGNAL( DbErased() ), this, SLOT( slotReConnect() ) );
  connect( this, SIGNAL( TableErased() ), this, SLOT( slotReConnect() ) );
  connect( this, SIGNAL( NewDbCreated() ), this, SLOT( slotReConnect() ) );
  connect( this, SIGNAL( NewTableCreated() ), this, SLOT( slotReConnect() ) );

//  lastDB = new QString;
//  lastTable = new QString;
  lastCols = NULL;

}

SqlGuiDoc::~SqlGuiDoc()
{
}

void SqlGuiDoc::addView(SqlGuiView *view)
{
  pViewList->append(view);
}

void SqlGuiDoc::removeView(SqlGuiView *view)
{
  pViewList->remove(view);
}
void SqlGuiDoc::setURL(const KURL &url)
{
  doc_url=url;
}

const KURL& SqlGuiDoc::URL() const
{
  return doc_url;
}

void SqlGuiDoc::slotUpdateAllViews(SqlGuiView *sender)
{
  SqlGuiView *w;
  if(pViewList)
  {
    for(w=pViewList->first(); w!=0; w=pViewList->next())
    {
      if(w!=sender)
        w->repaint();
    }
  }

}


QString* SqlGuiDoc::resultDB()
{
  return lastDB;
}

QString* SqlGuiDoc::resultTable()
{
  return lastTable;
}

spalten_t* SqlGuiDoc::resultCols()
{
  return lastCols;
}


void SqlGuiDoc::slotDbCreate( QString* db )
{

  if(db->isEmpty()) return;

  cout << "DB-Creating-Request..." << endl;

  cout << db->latin1() << endl;

  QString * q = new QString("CREATE DATABASE ");
  q->append( *db );
  
  MYSQL datenbank;
  mysql_init( &datenbank );
  MYSQL_RES * res;

  if(!mysql_real_connect( &datenbank, doc_host.latin1() , doc_uname.latin1() , doc_pass.latin1(), "", 0, NULL, 0)){
    cout << "Connection failed..." << endl;
    QString *errmsg = new QString( tr("Error connecting to server.") );
    emit( DbQueryError( errmsg) );
    return;
  }

  int okay = mysql_query( &datenbank, q->latin1() );
  
  if(okay==0){
    emit( NewDbCreated() );
  } else {
    QString *errmsg = new QString( tr("Error creating database. Maybe no permission?") );
    emit( DbQueryError( errmsg ) );
  }

  mysql_close( &datenbank );

  
}

void SqlGuiDoc::slotDbDelete( QString* db )
{

  if(db->isEmpty()) return;

  cout << "DB-Deletion-Request..." << endl;

  cout << db->latin1() << endl;

  QString * q = new QString("DROP DATABASE ");
  q->append( *db );
  
  MYSQL datenbank;
  mysql_init( &datenbank );
  MYSQL_RES * res;

  if(!mysql_real_connect( &datenbank, doc_host.latin1() , doc_uname.latin1() , doc_pass.latin1(), "", 0, NULL, 0)){
    cout << "Connection failed..." << endl;
    QString *errmsg = new QString( tr("Error connecting to server.") );
    emit( DbQueryError( errmsg) );
    return;
  }

  int okay = mysql_query( &datenbank, q->latin1() );
  
  if(okay==0){
    emit( DbErased() );
  } else {
    QString *errmsg = new QString( tr("Error removing database. Maybe no permission?") );
    emit( DbQueryError( errmsg ) );
  }

  mysql_close( &datenbank );
}

void SqlGuiDoc::slotTableDelete( QString* db, QString* tb )
{

  cout << "Got Table-Deletion-Request..." << endl;
  cout << db->latin1() << " " << tb->latin1() << endl;
  if(db->isEmpty()) return;
  if(tb->isEmpty()) return;

  QString * q = new QString("DROP TABLE ");
  q->append( *db );
  q->append( "." );
  q->append( *tb );
  
  MYSQL datenbank;
  mysql_init( &datenbank );
  MYSQL_RES * res;

  if(!mysql_real_connect( &datenbank, doc_host.latin1() , doc_uname.latin1() , doc_pass.latin1(), "", 0, NULL, 0)){
    cout << "Connection failed..." << endl;
    QString *errmsg = new QString( tr("Error connecting to server.") );
    emit( DbQueryError( errmsg) );
    return;
  }

  int okay = mysql_query( &datenbank, q->latin1() );
  
  if(okay==0){
    emit( TableErased() );
  } else {
    QString *errmsg = new QString( tr("Error removing table. Maybe no permission?") );
    emit( DbQueryError( errmsg ) );
  }

  mysql_close( &datenbank );
}

void SqlGuiDoc::slotTableCheck( QString* db, QString* tb )
{

  cout << "Got Table-Check-Request..." << endl;

  if(db->isEmpty()) return;
  if(tb->isEmpty()) return;

  QString * q = new QString("CHECK TABLE ");
  q->append( *db );
  q->append( "." );
  q->append( *tb );
  
  MYSQL datenbank;
  mysql_init( &datenbank );
  MYSQL_RES * res;

  if(!mysql_real_connect( &datenbank, doc_host.latin1() , doc_uname.latin1() , doc_pass.latin1(), "", 0, NULL, 0)){
    cout << "Connection failed..." << endl;
    QString *errmsg = new QString( tr("Error connecting to server.") );
    emit( DbQueryError( errmsg) );
    return;
  }

  int okay = mysql_query( &datenbank, q->latin1() );
  
  if(okay==0){
    emit( tableChecked() );
//    emit( resultChanged() );
  } else {
    QString *errmsg = new QString( tr("Error checking table. Maybe no such table?") );
    emit( DbQueryError( errmsg ) );
  }

  mysql_close( &datenbank );
}

void SqlGuiDoc::slotTableOptimize( QString* db, QString* tb )
{

  cout << "Got Table-Optimize-Request..." << endl;

  if(db->isEmpty()) return;
  if(tb->isEmpty()) return;

  QString * q = new QString("OPTIMIZE TABLE ");
  q->append( *db );
  q->append( "." );
  q->append( *tb );
  
  MYSQL datenbank;
  mysql_init( &datenbank );
  MYSQL_RES * res;

  if(!mysql_real_connect( &datenbank, doc_host.latin1() , doc_uname.latin1() , doc_pass.latin1(), "", 0, NULL, 0)){
    cout << "Connection failed..." << endl;
    QString *errmsg = new QString( tr("Error connecting to server.") );
    emit( DbQueryError( errmsg) );
    return;
  }

  int okay = mysql_query( &datenbank, q->latin1() );
  
  if(okay==0){
    emit( tableOptimized() );
    emit( resultChanged() );
  } else {
    QString *errmsg = new QString( tr("Error optimizing table. Maybe no such table?") );
    emit( DbQueryError( errmsg ) );
  }

  mysql_close( &datenbank );
}

void SqlGuiDoc::slotTableRepair( QString* db, QString* tb )
{

  cout << "Got Table-Repair-Request..." << endl;

  if(db->isEmpty()) return;
  if(tb->isEmpty()) return;

  QString * q = new QString("REPAIR TABLE ");
  q->append( *db );
  q->append( "." );
  q->append( *tb );
  
  MYSQL datenbank;
  mysql_init( &datenbank );
  MYSQL_RES * res;

  if(!mysql_real_connect( &datenbank, doc_host.latin1() , doc_uname.latin1() , doc_pass.latin1(), "", 0, NULL, 0)){
    cout << "Connection failed..." << endl;
    QString *errmsg = new QString( tr("Error connecting to server.") );
    emit( DbQueryError( errmsg) );
    return;
  }

  int okay = mysql_query( &datenbank, q->latin1() );
  
  if(okay==0){
    emit( tableRepaired() );
//    emit( resultChanged() );
  } else {
    QString *errmsg = new QString( tr("Error repairing table. Maybe no such table?") );
    emit( DbQueryError( errmsg ) );
  }

  mysql_close( &datenbank );
}

QString SqlGuiDoc::getDbFromSQL(QString * sql)
{

}

QString SqlGuiDoc::getTableFromSQL(QString * sql)
{
uint i=0;
uint l=0;
bool gotToken=false;
bool gotTable=false;
QString * token = new QString;
QString * table = new QString;
QString * data  = new QString;

 if(sql) l=sql->length();

 token->setLatin1("");
 table->setLatin1("");
 data->setLatin1("");

 // Wir mssen als erstes den String in seine Schlsselworte - getrennt durch Leerzeichen - aufspalten:
 while(i<l){
   char a = sql->at(i).latin1();
   if( a != ' ' ){
     gotToken=false;
     token->append(a);
   } else {
     gotToken=true;
   }
   // Haben wir ein Schlsselwort, dann mssen wir schauen, ob wir es kennen:
   if(gotToken){
     // Wir vergleichen nur GROSSBUCHSTABEN:
     token = &(token->upper());
     
     if( (token->compare("FROM")==0) || (token->compare("INTO")==0) || (token->compare("UPDATE")==0) || (token->compare("TABLE")==0) ){
       //
       // typ. Flle: 
       // SELECT * FROM table WHERE xyz LIKE 'bla'
       // SELECT * FROM database.table WHERE xyz LIKE 'bla'
       //
       // INSERT INTO table (xyz) values (bla-bla)
       // INSERT INTO database.table (xyz) values (bla-bla)
       //
       // UPDATE table SET xyz='bla'
       // UPDATE database.table SET xyz='bla'
       //
       bool go=true;
       QString * key = new QString;
       char y;
       bool gotDot=false;
       // Nchstes Token suchen:
       while(go){
	 i++;
	 y = sql->at(i).latin1();
	 // Wie wandeln nicht in Grobuchstaben um, denn wir wollen ja den Tabellen-Namen (der ist case-sensitiv).
	 if( y != ' ' ){
	   go=true;
	   key->append(y);
	   if( y == '.' ) gotDot=true;
	 }
	 else{
	   go=false;
	 }
       }
       // So, jetzt haben wir "Tabelle" oder "Datenbank.Tabelle":
       gotTable=true;
       if(gotDot){
	 // Aufspalten:
	 uint m = key->length();
	 uint n=0;
	 bool before=true;
	 while(n<m){
	   char b = key->at(n).latin1();
	   if( b != '.' ){
	     if(before) data->append(b);
	     else table->append(b);
	   }
	 }
       } else {
	 // Wir haben bereits die Tabelle:
	 table->setLatin1( key->latin1() );
       }
     }
     // Token zurcksetzen und Platz fr das nchste schaffen:
     token->setLatin1("");
   }
   // Weiter im Text:
   i++;
   if(gotTable)break;
 }
 return *table;
}

void SqlGuiDoc::slotUpdate(MYSQL_RES * res)
{
  return;
}

void SqlGuiDoc::clearResult()
{
  if( RESULT_Start ){ RESULT_Start = clearrw( &RESULT_Start ); }
  RESULT=NULL;
}

row_t * SqlGuiDoc::clearrw( row_t ** rw )
{
//  cout << "Request for removing Row..." << endl;
  if( *rw ){ 
    if ((*rw)->next) { (*rw)->next = clearrw( &( (*rw)->next ) ); }
    if ((*rw)->feld){ (*rw)->feld = clearfd( &( (*rw)->feld) ); }
    delete (*rw);
    *rw = NULL;
  }
  return NULL;
  
}
field_t * SqlGuiDoc::clearfd( field_t ** fd )
{
//  cout << "Request for removing Field..." << endl;
  if( *fd ){ 
    if ((*fd)->next) { (*fd)->next = clearfd( &( (*fd)->next ) ); }
    delete (*fd)->value;
    delete (*fd);
    *fd = NULL;
  }
  return NULL;
  
}

base_t* SqlGuiDoc::cleardb( base_t **  db)
{
//  cout << "Request for removing DB..." << endl;
  if( *db ){ 
    cout << "Clearing memory for DB " << (*db)->name << endl;
    if ((*db)->next) { (*db)->next = cleardb( &( (*db)->next ) ); }
    if ((*db)->table_list){ (*db)->table_list = cleartb( &( (*db)->table_list) ); }
//    cout << (*db)->name << endl;
    delete (*db)->name;
    delete (*db);
    *db = NULL;
  }
  return NULL;
}


table_t* SqlGuiDoc::cleartb( table_t ** tb )
{
//  cout << "Request for removing table..." << endl;
  if( *tb ){
    if ( (*tb)->next ) { (*tb)->next = cleartb( &((*tb)->next ) ); }
    if ( (*tb)->spalten_liste){ (*tb)->spalten_liste = clearsp( &( (*tb)->spalten_liste) ); }
//    cout << (*tb)->name << endl;
    delete (*tb)->name;
    //    delete (*tb)->type_n;
    delete (*tb);
    *tb = NULL;
  }
  return NULL;
}


spalten_t * SqlGuiDoc::clearsp( spalten_t ** sp)
{
//  cout << "Request for removing Column..." << endl;
  if( *sp ){
    if ( (*sp)->next) { (*sp)->next = clearsp( &( (*sp)->next ) ); }
//    cout << (*sp)->name << endl;
    delete (*sp)->name;
    delete (*sp)->type;
    delete (*sp)->extra_flags;
    delete (*sp);
    *sp = NULL;
  }
  return NULL;
}

void SqlGuiDoc::slotClearMemory(){
  cout << "Got request for clearing memory..." << endl;
  this->clearResult();
  cout << "Clearing done." << endl;
}

void SqlGuiDoc::slotDoQuery(QString * Base, QString* Anfrage)
{


//  cout << "DB: " << Base->latin1() << endl;
//  cout << "Query: " << Anfrage->latin1() << endl;
//  cout << "Host: " << doc_host.latin1() << endl;
//  cout << "User: " << doc_uname.latin1() << endl;
//  cout << "PW: " << doc_pass.latin1() << endl;

// return;

  MYSQL datenbank;
  mysql_init( &datenbank );
  MYSQL_RES * res;

  if(Base->isEmpty()){
    cout << "No base selected..." << endl;
    QString *errmsg = new QString( tr("No base selected.") );
    emit( DbQueryError( errmsg) );
    return;  
  }
  if(Anfrage->isEmpty()){
    cout << "No SQL-String..." << endl;
    QString *errmsg = new QString( tr("No SQL statement.") );
    emit( DbQueryError( errmsg) );
    return;  
  }

  cout << "DB: " << Base->latin1() << endl;
  cout << "SQL: " << Anfrage->latin1() << endl;


  if(!mysql_real_connect( &datenbank, doc_host.latin1() , doc_uname.latin1() , doc_pass.latin1(), "", 0, NULL, 0)){
    cout << "Connection failed..." << endl;
    QString *errmsg = new QString( tr("Error connecting to server.") );
    emit( DbQueryError( errmsg) );
    mysql_close(&datenbank);
    return;
  }
  if(mysql_select_db( &datenbank, Base->latin1() )){
    cout << "No such base..." << endl;
    QString *errmsg = new QString( tr("No such database.") );
    emit( DbQueryError( errmsg) );
    mysql_close(&datenbank);
    return;
  }
  //
  // unter umstnden mu hier noch die escape-Funktion eingebaut werden
  //
  int okay = mysql_query( &datenbank, Anfrage->latin1() );
  if(! (res = mysql_use_result( &datenbank )) ){
    cout << "Empty query..." << endl;
////    emit queryEmpty();
    if (okay){
      QString *errmsg = new QString( tr("Query failed.") );
      emit( DbQueryError( errmsg) );
    }
    if ( RESULT ){
      delete RESULT;
      RESULT = NULL;
    }
    if ( RESULT_Start ){
      delete RESULT_Start;
      RESULT_Start = NULL;
    }
    lastDB->setLatin1( Base->latin1() );
//    emit( resultChanged() );
//    emit( NewTableCreated() );
//    lastDB->setLatin1( Base->latin1() );


    lastTable->setLatin1("");
    QUERY->setLatin1( Anfrage->latin1() );
    STATUS->setLatin1("");
    mysql_close(&datenbank);
    return;
  }
  else
  {
		// Jetzt die letzte Tabelle aus dem SQL-String raussuchen:
		// Wir brauchen uns nur um die "select"-Statements zu kmmern,
		// da nur diese eine Anzeige erzeugen, welche wir durch ein
		// "update"-statement modifizieren wollen.

		QString *sql = new QString( Anfrage->simplifyWhiteSpace() );
		QChar   bst = *(new QChar);

    lastDB->setLatin1( Base->latin1() );
    				
 		cout << "Analysing SQL..." << sql->latin1() << endl;

 		// hier mu evtl. noch nachgebessert werden, falls der tabellenname auf select endet...
 		int select = sql->find("select ",0,false);
 				
 		if (select > -1)
 		{
 			// Es handelt sich um ein select-statement:
 			cout << "Select-Statement..." << endl;
 			// wir lschen die alte Variable:
 			lastTable->setLatin1("");
 			int from_pos = sql->find("from ",0,false);
 			if (from_pos > -1)
 			{
 				// Wir haben einen Tabellen-Namen nach dem From:
 				cout << "found from-expression... now looking for table..." << endl;
 				from_pos += 5; // das wort "from " hat nochmals 5 buchstaben...
 				int l = sql->length();
				bool weiter = true;
				int j = from_pos;
 				while ( weiter )
 				{
 					bst = sql->at(j);
 					lastTable->append( bst );
 					j++;
 					if(j==l){weiter=false;}
 					if(bst == " "){weiter=false;}
 				}
 				cout << "tablename: " << lastTable->latin1() << endl;
 			}
 		}
  }

// Spaltennamen auslesen:
  unsigned int num_fields;
  unsigned int r;
  spalten_t * column;
  MYSQL_FIELD *fields;
  num_fields = mysql_num_fields( res );
  fields = mysql_fetch_fields( res );
  for(r = 0; r < num_fields; r++){
    column = new spalten_t;
    spalten_t * column_alt;
    int p = strlen( fields[r].name );
    char * sp_name = new char[ p+1 ];
    if( p>0 ){
      sp_name = strcpy( sp_name, fields[r].name );
    }else{ 
      sp_name=NULL;
    }
    column->name=sp_name;
    if(r==0) lastCols=column;
    else column_alt->next=column;
    column_alt=column;
  }
  if( column ) column->next=NULL;

  // Datenstze auslesen:
  MYSQL_ROW row;
  unsigned int num_cols = mysql_field_count( &datenbank );

  if ( RESULT ){
    delete RESULT;
    RESULT = NULL;
  }
  if ( RESULT_Start ){
    clearResult();
    delete RESULT_Start;
    RESULT_Start = NULL;
  }

  row_t * hilf_row = NULL;

  while( (row = mysql_fetch_row( res )) ){

    row_t * rw = new row_t;
    rw->next = NULL;

    unsigned long * len = mysql_fetch_lengths(res);

    field_t * hilf_col = NULL;

    // alle Elemente der Reihe anhngen:
    for( unsigned int i = 0; i<num_cols; i++){
      char * rw_name = new char[ len[ i ]+1 ];
      field_t * col = new field_t;
      if( len[i] )strcpy( rw_name, row[i] );
      else rw_name=NULL;
      col->value = rw_name;
      if( hilf_col )
      {
 				hilf_col->next = col;
      }
      else
      {
				rw->feld = col;
      }
      hilf_col = col;
    }
    // Abbruchbedingung:
    if(hilf_col)hilf_col->next=NULL;
    // nchste Reihe:
    if (hilf_row){
      hilf_row->next = rw;
    }
    else
    {
      RESULT_Start = rw;
    }
    hilf_row = rw;
  }
  // Abbruchbedingung:
  if(hilf_row)hilf_row->next=NULL;

  mysql_close( &datenbank );
  emit( resultChanged() );

//
//
// Mysql...?!? Grrrrrrr...!!!
// Da ist es echt effektiver, Klopapier beidseitig zu benutzen!
//
// Das hier sorgt fr nen fetten Absturz, irgendwo, irgenwann.
// Kaum nachvollziehbar, wieso, warum, weswegen:
//
//

//	if( ! ( lastTable->isEmpty() )  &&  ! ( lastDB->isEmpty() )  )
//	{
//  	if(!mysql_real_connect( &datenbank, doc_host.latin1() , doc_uname.latin1() , doc_pass.latin1(), "", 0, NULL, 0)){
//    	cout << "Connection failed..." << endl;
//	    QString *errmsg = new QString( tr("Error connecting to server.") );
//    	return;
//	  }
//	  else
//	  {
//			// Wir fragen MySQL nach dem Primary Key
//    	//
//    	// ...sollten wir besser lassen...
//    	MYSQL_ROW columns;
//	    MYSQL_RES * cols;
//			
//			cout << "Now selecting db " << lastDB->latin1() << endl;
//		
//	    char * query = new char[ 16 + lastTable->length() ];
//  	  int okay = mysql_select_db( &datenbank, lastDB->latin1() );
//  		query = strcpy( query, "SHOW INDEX FROM ");
//		  query = strcat(query, lastTable->latin1() );
//		
//			cout << "Okay, now finding keys: " << query << endl;
//		
//		  okay = mysql_query( &datenbank, query );
//	  	cout << okay << endl;
//			
//	  	if (!okay)
//	  	{
//	  		emit queryEmpty();
//		  	return;	
//	  	}
//	  	
//  	  cols = mysql_use_result( &datenbank );
//	  	primKey->setLatin1("");
//		
//			cout << "Okay, got keys" << endl;
//	  	bool weiter = true;
//	    while( (columns = mysql_fetch_row( cols ) ) && weiter )
//  	  {
//    	  char * key_name = new char[ strlen( columns[2] ) +1 ];
//	      strcpy( key_name, columns[2] );
//  	    char * col_name = new char[ strlen( columns[4] ) +1 ];
//    	  strcpy( col_name, columns[4] );
//		
//  	    cout << "okay, got a row "<< endl;
//		
//				if ( (key_name[0] == 'P') && (key_name[1] == 'R') && (key_name[2] == 'I') && (key_name[3] == 'M') && (key_name[4] == 'A') && (key_name[5] == 'R') && (key_name[6] == 'Y') )
//				{
//					cout << col_name << endl;
//					primKey->setLatin1( col_name );
//					weiter = false;
//				}
//			
//			}	
//		}
//	  mysql_close( &datenbank );
//	}
//	else
//	{
//		primKey = NULL;
//	}
//	cout << "found primary key: " << primKey->latin1() << endl;
	
//
//
// bis hierher. Ohne das da oben wre alles kein Problem!
//
//


// Also lesen wir die Sachen aus den bereits eingescannten Daten aus...
// (...geht eh viel schneller...)
	spalten_t * sp;
	table_t   * tab;
	base_t    * bse = DB;
	while( bse ){
		if ( strcmp(bse->name, lastDB->latin1() ) == 0 )
		{
			tab = bse->table_list;
			cout << "Found last db in DB-Object " << endl;
			bse = NULL;
			while( tab )
			{
				if ( strcmp( tab->name, lastTable->latin1() ) == 0 )
				{	
					sp = tab->spalten_liste;
					cout << "Found last table in DB-Object " << endl;
					tab = NULL;
					while(sp){
						if ( sp->pri == 'Y' ){
							cout << "Found primary-column in last table " << endl;
							primKey->setLatin1( sp->name );
							cout << primKey->latin1() << endl;
							sp = NULL;
						}
						else
						{
							sp = sp->next;
						}
					}
				}	
				else
				{	
					tab = tab->next;
				}
			}	
		} else {
			bse = bse->next;
		}
	}
	emit lastActionChanged( lastDB, lastTable, primKey);
}

bool SqlGuiDoc::saveModified()
{
  bool completed=true;

  if(modified)
  {
//    SqlGuiApp *win=(SqlGuiApp *) parent();
//    int want_save = KMessageBox::warningYesNoCancel(win, i18n("Warning"),i18n("The current file has been modified.\nDo you want to save it?"));
//    switch(want_save)
//    {
//      case 1:
//           if (doc_url.fileName() == i18n("Untitled"))
//           {
//             win->slotFileSaveAs();
//           }
//           else
//           {
//             saveDocument(URL());
//       	   };
//
//       	   deleteContents();
//           completed=true;
//           break;
//
//      case 2:
//           setModified(false);
//           deleteContents();
//           completed=true;
//           break;	
//
//      case 3:
//           completed=false;
//           break;
//
//      default:
//           completed=false;
//           break;
//    }
  }

  return completed;
}

void SqlGuiDoc::closeDocument()
{
  deleteContents();
}

bool SqlGuiDoc::newDocument()
{
  /////////////////////////////////////////////////
  // TODO: Add your document initialization code here
  /////////////////////////////////////////////////
  modified=false;
  doc_url.setFileName(i18n("Not connected"));

  return true;
}

void SqlGuiDoc::slotReConnect()
{
  cout << "Reconnecting... " << endl; // << doc_host << "  " << doc_uname << "  " << doc_pass << endl;
  QString *h = new QString( doc_host );
  QString *u = new QString( doc_uname );
  QString *p = new QString( doc_pass );
  this->openDocument( *h, *u, *p );
 
}

void SqlGuiDoc::slotDoLogin( QString url, QString uname, QString pwd)
{
  this->openDocument(url,uname,pwd);
}

bool SqlGuiDoc::openDocument(const QString& url, const QString& uname, const QString& pwd)
{
  const char * host;
  const char * user;
  const char * pass;
  char * info;
  int anz_db  = 0;
  int anz_tab = 0;
  SqlGuiApp * myApp = (SqlGuiApp *) parent();

  QApplication::setOverrideCursor( Qt::waitCursor );

  if ( ! url.isEmpty() ){

    cout << "Aufruf von openDocument" << endl;
    
    host = url.latin1();
    user = uname.latin1();
    pass = pwd.latin1();
	  
    doc_host.setLatin1(host);
    doc_uname.setLatin1(user);
    doc_pass.setLatin1(pass);

    unsigned int num_rows;
    unsigned int i;

    MYSQL datenbank;
    mysql_init( &datenbank );
    if(! &datenbank ){
      cout << "ERROR initializing MYSQL-Object... :-(" << endl;
      delete DB;
      DB=NULL;
      myApp->slotStatusMsg(i18n("Not connected..."));
    } else{
      mysql_real_connect(&datenbank, host , user , pass, "", 0, NULL, 0);
      if(! &datenbank ){ 
	cout << "ERROR connecting to server... :-((" << endl;
	QString *errmsg = new QString( tr("Error connecting to server.") );
	QApplication::restoreOverrideCursor();
	emit( DbQueryError( errmsg) );
	delete DB;
	DB=NULL;
	myApp->slotStatusMsg(i18n("Not connected..."));
      } else {

	info = mysql_get_server_info(&datenbank);
	cout << "MySQL-Info: " << info << endl;
	doc_url.setFileName(i18n("Connected"));

	MYSQL_RES * result;
	result = mysql_list_dbs( &datenbank, "%");

	if ( ! result ){
	  cout << "ERROR getting DBs... :-(" << endl;
	  QString *errmsg = new QString( tr("Error getting databases...\nMaybe the server is busy or down...") );
	  QApplication::restoreOverrideCursor();
	  emit( DbQueryError( errmsg) );
	  delete DB;
	  DB=NULL;
	  myApp->slotStatusMsg(i18n("Not connected..."));
	} else {

//	  num_rows = mysql_num_rows(result);
//	  cout << "Got " << num_rows << " DBs..." << endl;
	  
	  MYSQL_ROW row;

	  clearResult();
	  DB=NULL;
	  base_t * hilf=NULL;
	  
	  while( (row = mysql_fetch_row(result)) ){
//	    cout << row[0] << " ";
	    // Fr jede DB erzeugen wir einen neuen eintrag im Objekt DB:
	    base_t * temp = new base_t;
	    char * db_name=NULL;
	    db_name = new char[ strlen(row[0])+1 ];
	    db_name = strcpy( db_name , row[0]);
	    temp->name = db_name;
	    // eigentlich knnten wir hier auch schon alle Tabellen fr die jeweilige DB auslesen,
	    // aber das gibt Probleme bei den verschachtelten MySQL-Aufrufen... deswegen lesen
	    // wir die Tabellen spter separat aus...
	    if (DB==NULL){
	      DB = temp;
	    } else {
	      hilf->next = temp;
	    }

	    hilf = temp;
	  }
	  hilf->next=NULL;
//	  cout << endl;
	  mysql_free_result( result );
	}
	myApp->slotStatusMsg(i18n("Connected..."));

	mysql_close( &datenbank );
	cout << "Connection is closed..." << endl;
      }
    }

    // Jetzt die Tabellen auslesen:
    mysql_init( &datenbank );
    MYSQL_RES * dbs;
    if(! &datenbank ){
      cout << "ERROR initializing MYSQL-Object... :-(" << endl;
      delete DB;
      DB=NULL;
      myApp->slotStatusMsg(i18n("Not connected..."));
    } else{
      MYSQL* datenbank_p = mysql_real_connect(&datenbank, host , user , pass, "", 0, NULL, 0);
      if(! datenbank_p ){
	cout << "ERROR connecting to server... :-((" << endl;
	delete DB;
	DB=NULL;
	myApp->slotStatusMsg(i18n("Not connected..."));
      } else {
	base_t * test = DB;
	cout << "Lese Tabellen aus..." << endl;
	while( test ){
	  int okay = mysql_select_db( &datenbank, test->name);
//	  cout << endl << "Selected " << test->name << ", Status: " << okay << endl;
		dbs = NULL;
		if(okay==0){  dbs = mysql_list_tables( &datenbank, "%");}
//	  cout << mysql_error( &datenbank );
//	  cout << mysql_error( &datenbank );
//	  unsigned int num_tabs = mysql_num_rows( dbs );
//	  cout << "DB: " << test->name << ": "<< "Got " << num_tabs << " Tables..." << endl;

	  MYSQL_ROW tables;
//	  cout << "Tabellen eingelesen..." << endl;
	  table_t * alt = NULL;
	  test->table_list = NULL;
		if(dbs){
	  while( (tables = mysql_fetch_row( dbs )) ){
//	    cout << tables[0] << " ";

	    char * tab_name = new char[ strlen( tables[0] ) + 1 ];
	    strcpy(tab_name , tables[0] );

	    table_t* test3 = new table_t;
	    test3->name = tab_name;

	    if(alt){
	      alt->next=test3;
	    } else {
	      test->table_list = test3;
	    }
	    alt = test3;
	  }
	  }
	  if(alt)alt->next=NULL;
	  if(dbs){mysql_free_result( dbs );}
	  if(test)test = test->next;
	
	}
	mysql_close(&datenbank);
	myApp->slotStatusMsg(i18n("Successfull connection..."));
//	cout << endl;
	mysql_close( & datenbank );
      }
    }


    mysql_init( &datenbank );
    MYSQL_RES * cols;
    if(! &datenbank ){
      cout << "ERROR initializing MYSQL-Object... :-(" << endl;
      delete DB;
      DB=NULL;
      myApp->slotStatusMsg(i18n("Not connected..."));
    } else{
      MYSQL* datenbank_p = mysql_real_connect(&datenbank, host , user , pass, "", 0, NULL, 0);
      if(! datenbank_p ){
	cout << "ERROR connecting to server... :-((" << endl;
  	delete DB;
	DB=NULL;
	myApp->slotStatusMsg(i18n("Not connected..."));
      } else {

	//
	// Kann jemand mal dieses dumme Api dazu bringen, verschachtelte Abfragen durchzufhren?
	//
	// etwa so:
	//
	// abfrage1 = list_dbs();
	// while ( db = fetch_row( abfrage1) ){
	//   abfrage2 = list_tables( db );
	//   while( tb = fetch_row( abfrage2 ) ){
	//     abfrage3 = list_fields( tb );
	//     while( fd = fetch_row( abfrage3 ) ){
	//       cout << bla-bla...
	//     }
	//   }
	// }
	//
	// Welche Rolle spielen dabei use_result und store_result?
	//


	// Jetzt die Spalten der Tabellen raussuchen...
	//
	base_t  * bse = DB;
	table_t * tbl = bse->table_list;
	while(bse){
	  tbl = bse->table_list;
	  while( tbl ){
	    MYSQL_ROW columns;
	    char * query = new char[ 20 + strlen(tbl->name) ];
	    int okay = mysql_select_db( &datenbank, bse->name);
//	    cout << mysql_error( &datenbank );
	    query = strcpy( query, "SHOW COLUMNS FROM ");
	    query = strcat(query, tbl->name );
	    
	    okay = mysql_query( &datenbank, query );
	    cols = mysql_use_result( &datenbank );
//	    cout << "DB: " << bse->name  << endl << query << endl << mysql_error( &datenbank ) << endl;
//	    cout << "Selected Table: " << tbl->name << endl;
	    spalten_t * alt2 = NULL;
	    while( (columns = mysql_fetch_row( cols ) ) ){
	      char * sp_name = new char[ strlen( columns[0] ) +1 ];
	      strcpy( sp_name, columns[0] );
	      char * sp_type = new char[ strlen( columns[1] ) +1 ];
	      strcpy( sp_type, columns[1] );
	      char * sp_extra = new char[ strlen( columns[5] ) +1 ];
	      strcpy( sp_extra, columns[5] );
//	      cout << columns[0] << " " << columns[1] << " " << columns[2] << endl;
	      spalten_t * spl = new spalten_t;
	      spl->name = sp_name;
	      spl->type = sp_type;
	      spl->extra_flags = sp_extra;
	      if( (columns[3][0] == 'P') && (columns[3][1] == 'R') && (columns[3][2] == 'I') ){
					spl->pri='Y';
	      }else{
					spl->pri=' ';
	      }

	      spl->next=NULL;
//	      cout << sp_name << " ";
	      if( alt2 ){
					alt2->next = spl;
	      } else {
					tbl->spalten_liste = spl;
	      }
	      alt2 = spl;
	    }
	    mysql_free_result( cols );
	    if(tbl)tbl = tbl->next;
	  }
	  if(bse)bse=bse->next;
	}
      }
      mysql_close(&datenbank);
    }    
  }
  modified=false;
  emit( connectionChanged() );

  if ( RESULT ){
    delete RESULT;
    RESULT = NULL;
  }
  if ( RESULT_Start ){
    delete RESULT_Start;
    RESULT_Start = NULL;
  }

//  emit( resultChanged() );
  emit loginComplete();
  QApplication::restoreOverrideCursor();
  return true;
}

QString * SqlGuiDoc::getPrimaryKey(){
	return primKey;
}

QString SqlGuiDoc::getHost(){
	return doc_host;
}
QString SqlGuiDoc::getUser(){
	return doc_uname;
}
QString SqlGuiDoc::getPass(){
	return doc_pass;
}

QString * SqlGuiDoc::getLastTable(){
	return lastTable;
}

bool SqlGuiDoc::saveDocument(const KURL& url, const char *format /*=0*/)
{
  /////////////////////////////////////////////////
  // TODO: Add your document saving code here
  /////////////////////////////////////////////////

  modified=false;
  return true;
}

void SqlGuiDoc::deleteContents()
{
  /////////////////////////////////////////////////
  // TODO: Add implementation to delete the document contents
  /////////////////////////////////////////////////
}
